package com.pqs9300.mapper.uploadFiles;

import com.pqs9300.pojo.transaction.OverRunLog;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * @Author: xuyang
 * @Description:
 */
public interface UploadFilesMapper {
    String getOverRunLogName(@Param("supIndex")String supIndex,@Param("lineIndex")String lineIndex,@Param("updateTime") Date updateTime);

    Integer updateFeedbackUrl(@Param("filePath") String filePath, @Param("supIndex")String supIndex,@Param("lineIndex")String lineIndex, @Param("updateTime")Date updateTime);
    Integer updateTestReportUrl(@Param("filePath")String filePath, @Param("supIndex")String supIndex,@Param("lineIndex")String lineIndex, @Param("updateTime")Date updateTime);
    Integer updateReviseFeedbackUrl(@Param("filePath")String filePath, @Param("supIndex")String supIndex,@Param("lineIndex")String lineIndex, @Param("updateTime")Date updateTime);

    //返回当前监督需要上传文件所有名字
    List<String> getAllFileName(@Param("supIndex")String supIndex,@Param("stepCount")Integer stepCount);

    //获取附表文件上传路径，判断是否需要改变文件状态
    List<OverRunLog> getOverRunLog(@Param("supIndex")String supIndex);

    Integer updateFilesFeedbackUrl(@Param("filePath") String filePath, @Param("fileName") String fileName, @Param("supIndex")String supIndex);
    Integer updateFilesTestReportUrl(@Param("filePath")String filePath, @Param("fileName") String fileName, @Param("supIndex")String supIndex);
    Integer updateFilesReviseFeedbackUrl(@Param("filePath")String filePath, @Param("fileName") String fileName, @Param("supIndex")String supIndex);

    Integer updateProgress(@Param("supIndex")String supIndex,@Param("progress")Integer progress );

    Integer getSuperviseType(@Param("supIndex")String supIndex);
}
